﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using System.Reflection;
using System.Diagnostics;

namespace round5Visualizer
{
    class Executor
    {
        private string command;
        private string startFilePath;
        private string endFilePath;
        private int numberOfOperations;
        private int maxNumberOfPixelsChanged;
        private string solutionOutputFile = "mytask.out";
        private string[] listOfImages;

        public Executor()
        {
            command = "";
            startFilePath = "";
            endFilePath = "";
            numberOfOperations = -1;

            maxNumberOfPixelsChanged = 0;
        }

        public void setCommand(string command)
        {
            this.command = command;
        }

        public void setStartFilePath(string startFilePath)
        {
            this.startFilePath = startFilePath;
        }
        public void setListOfImages(string[] listOfImages)
        {
            this.listOfImages = listOfImages;
        }
        public string[] getListOfImages()
        {
            return listOfImages;
        }

        public string getStartFilePath()
        {
            return startFilePath;
        }

        public string getEndFilePath()
        {
            return endFilePath;
        }

        
        public void setEndFilePath(string endFilePath)
        {
            this.endFilePath = endFilePath;
        }

        public void setNumberOfOperations(int numberOfOperations)
        {
            this.numberOfOperations = numberOfOperations;
        }


        public int getNumberOfOperations()
        {
            return numberOfOperations;
        }
        
        //CreateInputFile
        public bool createInputFile(string inputFileName)
        {
            Bitmap startImage = new Bitmap(startFilePath);
            Bitmap endImage = new Bitmap(endFilePath);
            if (startImage.Height != endImage.Height || startImage.Width != endImage.Width)
            {
                MessageBox.Show("Images have different sizes", "Input file generation failed!",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
                return false;
            }
            File.Delete(inputFileName);
            using (System.IO.StreamWriter file = new System.IO.StreamWriter(inputFileName, true))
            {
                int n = startImage.Height, m = startImage.Width;

                file.WriteLine(n + " " + m + " " + numberOfOperations);

                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < m; ++j)
                        file.Write(startImage.GetPixel(j, i).R + " ");
                    file.Write("\n");
                }
                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < m; ++j)
                        file.Write(startImage.GetPixel(j, i).G + " ");
                    file.Write("\n");
                }
                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < m; ++j)
                        file.Write(startImage.GetPixel(j, i).B + " ");
                    file.Write("\n");
                }

                n = endImage.Height;
                m = endImage.Width;

                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < m; ++j)
                        file.Write(endImage.GetPixel(j, i).R + " ");
                    file.Write("\n");
                }
                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < m; ++j)
                        file.Write(endImage.GetPixel(j, i).G + " ");
                    file.Write("\n");
                }
                for (int i = 0; i < n; ++i)
                {
                    for (int j = 0; j < m; ++j)
                        file.Write(endImage.GetPixel(j, i).B + " ");
                    file.Write("\n");
                }
                file.Close();
                return true;
            }
        }


        private bool overlap(int x1, int y1, int x2, int y2, int x3, int y3, int x4, int y4)
        {
            if (x2 < x3)
                return false;
            if (x4 < x1)
                return false;
            if (y2 < y3)
                return false;
            if (y4 < y1)
                return false;

            return true;
        }

        

        private double distance(Color t1, Color t2)
        {
            double r = t1.R - t2.R;
            double g = t1.G - t2.G;
            double b = t1.B - t2.B;

            return Math.Sqrt(r * r + g * g + b * b);
        }
        double getScore(Bitmap bp)
        {
            double res = 0;
            Bitmap end = new Bitmap(endFilePath);
            int N = end.Width, M = end.Height;

            for (int i = 0; i < N; ++i)
                for (int j = 0; j < M; ++j)
                {
                    res += distance(bp.GetPixel(i, j), end.GetPixel(i, j));
                }
            return res;

        }

        //run the solution
        public bool runSolution(string input, string output)
        {
            Process m_command = new Process();
            m_command.StartInfo.FileName ="cmd.exe";
            m_command.StartInfo.UseShellExecute = false;
            m_command.StartInfo.WindowStyle = ProcessWindowStyle.Normal;
//            m_command.StartInfo.CreateNoWindow = true;
           // m_command.StartInfo.RedirectStandardInput = true;
            //m_command.StartInfo.RedirectStandardOutput = true;
            m_command.StartInfo.Arguments = @"/C " + command + " < " + input + " > " + output;
            m_command.Start();


            
            m_command.WaitForExit();
            //check if error level = ?;

            return true;
        }
        
        //Visualize text file
        public double visualizeSolution(Form1 gui, string output, bool isMultiple)
        {
            solutionOutputFile = output;

            Random rnd = new Random();
            Bitmap img = new Bitmap(startFilePath);
            string inp = System.IO.File.ReadAllText(solutionOutputFile);
            string[] operations = inp.Split('\n');
            int iter = gui.getRefreshRate() / 10;
            int cnt = 0;
            Form1.SetControlPropertyThreadSafe(gui.getProgress(), "Text", System.Convert.ToString(cnt));
            int oper = int.Parse(operations[0]), idx = 1;
            maxNumberOfPixelsChanged = img.Height * img.Width * 100;

            gui.getPictureBox2().Image = new Bitmap(endFilePath);

            if (oper > int.Parse(gui.getTextBox1().Text))
            {
                if (isMultiple == false)
                {
                    MessageBox.Show("Too much operations", "Error!",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                return -1.0;
            }

            while (oper > 0)
            {
                for (int i = 0; i < gui.getRefreshRate(); ++i)
                {
                    --oper;
                    ++cnt;
                    if (doOperation(img, operations[idx++]) == false)
                    {
                        if (isMultiple == false)
                        {
                            MessageBox.Show("Invalid operation!", "Error!",
                            MessageBoxButtons.OK, MessageBoxIcon.Information);
                        }
                        return -1.0;
                    }
                    Thread.Sleep(1000 / gui.getRefreshRate());
                    if (i % iter == 0)
                        gui.getPictureBox1().Image = new Bitmap(img);
                    if (oper == 0)
                        break;
                };
                gui.getPictureBox1().Image = new Bitmap(img);
                Form1.SetControlPropertyThreadSafe(gui.getProgress(), "Text", System.Convert.ToString(cnt));
            }

            return getScore(img);
        }

        //simulate one operation
        private bool doOperation(Bitmap bt, string oper)
        {
            string[] arr = oper.Split(new string[] { " " }, StringSplitOptions.RemoveEmptyEntries);
            if (arr.Length != 6)
                return false;
            int x1 = int.Parse(arr[0]), y1 = int.Parse(arr[1]), x2 = int.Parse(arr[2]), y2 = int.Parse(arr[3]);
            int szx = int.Parse(arr[4]), szy = int.Parse(arr[5]);
            if (x1 + szx > bt.Width && y1 + szy > bt.Height)
                return false;
            if (overlap(x1, y1, x1 + szx - 1, y1 + szy - 1, x2, y2, x2 + szx - 1, y2 + szy - 1) == true)
                return false;

            for (int i = 0; i < szx; ++i)
                for (int j = 0; j < szy; ++j)
                {
                    Color from = bt.GetPixel(y2 + j, x2 + i), to = bt.GetPixel(y1 + j, x1 + i);
                    int fromR = from.R, fromG = from.G, fromB = from.B;
                    int toR = to.R, toG = to.G, toB = to.B;
                    int nwR = (fromR + toR) / 2, nwG = (fromG + toG) / 2, nwB = (fromB + toB) / 2;
                    bt.SetPixel(y1 + j, x1 + i, Color.FromArgb(nwR, nwG, nwB));
                }
            maxNumberOfPixelsChanged -= (szx * szy);
            if (maxNumberOfPixelsChanged < 0)
                return false;
            return true;
        }
    }
}
